@@ -9,12 +9,20 @@
 //! - Useful API methods: e.g. `find`, `filter`, `has`, `is`, `not`, `add`, `closest`, `map` and so on.
 //! - Content mutations: `set_html`, `set_text` can also called by text nodes, external methods such as `append_text`、`prepend_text` are supported.
 //! - Well tested: the unit tests have covered most cases, but if you meet any bugs or questions, welcome to submit issues or PR to us.
-
+#[macro_use]
+mod macros;
 mod mesdoc;
+// feature="text"
+cfg_feat_text! {
+	use mesdoc::interface::Texts;
+}
+// feature="mutation"
+cfg_feat_insertion! {
+	use mesdoc::interface::InsertPosition;
+}
 use mesdoc::interface::{
 	BoxDynElement, BoxDynNode, BoxDynText, BoxDynUncareNode, Elements, IDocumentTrait, IElementTrait,
-	IErrorHandle, IFormValue, INodeTrait, ITextTrait, IUncareNodeTrait, InsertPosition, MaybeDoc,
-	MaybeElement, Texts,
+	IErrorHandle, IFormValue, INodeTrait, ITextTrait, IUncareNodeTrait, MaybeDoc, MaybeElement,
 };
 
 use mesdoc::utils::is_equal_chars;
@@ -30,10 +38,14 @@ use std::rc::Rc;
 use std::{any::Any, cell::RefCell};
 // re export `IAttrValue` `IEnumTyped` `INodeType`
 pub mod types {
+	// text
+	cfg_feat_text! {
+		pub use crate::mesdoc::interface::Texts;
+	}
 	pub use crate::mesdoc::error::BoxDynError;
 	pub use crate::mesdoc::interface::{
 		BoxDynElement, BoxDynNode, BoxDynText, Elements, IAttrValue, IDocumentTrait, IEnumTyped,
-		IFormValue, INodeType, Texts,
+		IFormValue, INodeType,
 	};
 	pub use crate::mesdoc::selector::Combinator;
 }
@@ -57,38 +69,48 @@ impl Dom {
 			}));
 		}
 	}
-
-	fn validate_dom_change(dom: &Rc<RefCell<Node>>, node: &BoxDynElement, method: &str) -> bool {
-		// test if current node is element node
-		let my_node_type = dom.borrow().node_type;
-		if my_node_type != NodeType::Tag {
-			Dom::halt(
-				dom,
-				method,
-				&format!("Can't {} for a {:?} type", method, my_node_type),
-			);
-			return false;
-		}
-		// document
-		if let INodeType::Document = node.node_type() {
-			Dom::halt(dom, method, &format!("Can't {} of a document type", method));
-			return false;
-		}
-		// test if same node
-		if dom.is(node) {
-			Dom::halt(dom, method, &format!("Can't {} of dom.", method));
-			return false;
-		}
-		// test if the node is dom's parent node
-		let mut cur = dom.cloned();
-		while let Some(parent) = cur.parent() {
-			if parent.is(node) {
-				Dom::halt(dom, method, &format!("Can't {} of self's parent", method));
+	// when mutation feature is open
+	cfg_feat_insertion! {
+		fn validate_dom_change(dom: &Rc<RefCell<Node>>, node: &BoxDynElement, method: &str) -> bool {
+			// test if current node is element node
+			let my_node_type = dom.borrow().node_type;
+			if my_node_type != NodeType::Tag {
+				Dom::halt(
+					dom,
+					method,
+					&format!("Can't {} for a {:?} type", method, my_node_type),
+				);
 				return false;
 			}
-			cur = parent;
+			// document
+			if let INodeType::Document = node.node_type() {
+				Dom::halt(dom, method, &format!("Can't {} a document type", method));
+				return false;
+			}
+			// test if same node
+			if dom.is(node) {
+				Dom::halt(
+					dom,
+					method,
+					&format!("Can't {} a dom that contains itself.", method),
+				);
+				return false;
+			}
+			// test if the node is dom's parent node
+			let mut cur = dom.cloned();
+			while let Some(parent) = cur.parent() {
+				if parent.is(node) {
+					Dom::halt(
+						dom,
+						method,
+						&format!("Can't {} a dom that contains it's parent", method),
+					);
+					return false;
+				}
+				cur = parent;
+			}
+			true
 		}
-		true
 	}
 }
 
@@ -359,39 +381,40 @@ impl INodeTrait for Rc<RefCell<Node>> {
 }
 
 impl ITextTrait for Rc<RefCell<Node>> {
-	/// Remove a text node.
-	///
-	fn remove(self: Box<Self>) {
-		let index = self.index();
-		if let Some(parent) = &self.borrow_mut().parent {
-			if let Some(parent) = parent.upgrade() {
-				if let Some(childs) = &mut parent.borrow_mut().childs {
-					// remove the text node
-					childs.remove(index);
-					// change next siblings index
-					reset_next_siblings_index(index, &childs[index..]);
+	cfg_feat_text! {
+		/// Remove a text node.
+		fn remove(self: Box<Self>) {
+			let index = self.index();
+			if let Some(parent) = &self.borrow_mut().parent {
+				if let Some(parent) = parent.upgrade() {
+					if let Some(childs) = &mut parent.borrow_mut().childs {
+						// remove the text node
+						childs.remove(index);
+						// change next siblings index
+						reset_next_siblings_index(index, &childs[index..]);
+					}
 				}
 			}
 		}
-	}
 
-	// append text
-	fn append_text(&mut self, content: &str) {
-		let chars = content.chars().collect::<Vec<char>>();
-		if let Some(content) = &mut self.borrow_mut().content {
-			content.extend(chars);
-		} else {
-			self.borrow_mut().content = Some(chars);
+		// append text
+		fn append_text(&mut self, content: &str) {
+			let chars = content.chars().collect::<Vec<char>>();
+			if let Some(content) = &mut self.borrow_mut().content {
+				content.extend(chars);
+			} else {
+				self.borrow_mut().content = Some(chars);
+			}
 		}
-	}
 
-	// prepend text
-	fn prepend_text(&mut self, content: &str) {
-		let chars = content.chars().collect::<Vec<char>>();
-		if let Some(content) = &mut self.borrow_mut().content {
-			content.splice(0..0, chars);
-		} else {
-			self.borrow_mut().content = Some(chars);
+		// prepend text
+		fn prepend_text(&mut self, content: &str) {
+			let chars = content.chars().collect::<Vec<char>>();
+			if let Some(content) = &mut self.borrow_mut().content {
+				content.splice(0..0, chars);
+			} else {
+				self.borrow_mut().content = Some(chars);
+			}
 		}
 	}
 }
@@ -712,215 +735,223 @@ impl IElementTrait for Rc<RefCell<Node>> {
 			.collect::<String>()
 	}
 
-	/// impl `remov_child`
-	fn remove_child(&mut self, ele: BoxDynElement) {
-		if let Some(parent) = &ele.parent() {
-			if self.is(parent) {
-				// is a child
-				if let Some(childs) = self.borrow_mut().childs.as_mut() {
-					let index = ele.index();
-					// if not the last child
-					if index != childs.len() - 1 {
-						reset_next_siblings_index(index, &childs[index + 1..]);
+	// when the feature `destory` or `insertion` is open
+	cfg_feat_mutation! {
+		/// impl `remov_child`
+		fn remove_child(&mut self, ele: BoxDynElement) {
+			if let Some(parent) = &ele.parent() {
+				if self.is(parent) {
+					// is a child
+					if let Some(childs) = self.borrow_mut().childs.as_mut() {
+						let index = ele.index();
+						// if not the last child
+						if index != childs.len() - 1 {
+							reset_next_siblings_index(index, &childs[index + 1..]);
+						}
+						// remove child
+						childs.remove(index);
 					}
-					// remove child
-					childs.remove(index);
 				}
 			}
 		}
 	}
-	// append child
-	fn insert_adjacent(&mut self, position: &InsertPosition, node: &BoxDynElement) {
-		// base validate
-		let action = position.action();
-		if !Dom::validate_dom_change(self, node, action) {
-			return;
-		}
-		let node_type = node.node_type();
-		let specified: Box<dyn Any> = node.cloned().to_node();
-		if let Ok(dom) = specified.downcast::<RefNode>() {
-			// get the nodes
-			let mut nodes = match node_type {
-				INodeType::DocumentFragement => {
-					if let Some(childs) = &dom.borrow().childs {
-						childs.iter().map(Rc::clone).collect::<Vec<RefNode>>()
-					} else {
-						vec![]
-					}
-				}
-				_ => {
-					// remove current node from parent's childs
-					if let Some(parent) = &mut node.parent() {
-						parent.remove_child(node.cloned());
-					}
-					vec![*dom]
-				}
-			};
-			// filter the node allowed
-			let tag_name = self.tag_names();
-			// remove not allowed nodes
-			remove_not_allowed_nodes(&tag_name, &mut nodes);
-			// check if is empty
-			if nodes.is_empty() {
+	// when the feature `insertion` is open
+	cfg_feat_insertion! {
+		// append child
+		fn insert_adjacent(&mut self, position: &InsertPosition, node: &BoxDynElement) {
+			// base validate
+			let action = position.action();
+			if !Dom::validate_dom_change(self, node, action) {
 				return;
 			}
-			// insert
-			use InsertPosition::*;
-			match position {
-				BeforeBegin | AfterEnd => {
-					// get index first, for borrow check
-					let mut index = self.index();
-					let mut nexts: Vec<RefNode> = vec![];
-					let insert_len = nodes.len();
-					// it's insertAfter, increase the insertion index
-					if *position == AfterEnd {
-						index += 1;
-					}
-					// always reset node indexs
-					reset_next_siblings_index(index, &nodes);
-					// split to prev and next
-					if let Some(parent) = &self.borrow_mut().parent {
-						if let Some(parent) = &parent.upgrade() {
-							if let Some(childs) = &mut parent.borrow_mut().childs {
-								// split the nexts for reset index.
-								if index < childs.len() {
-									nexts = childs.split_off(index);
-								}
-								// set node parent
-								for node in &nodes {
-									node.borrow_mut().parent = Some(Rc::downgrade(parent));
-								}
-								// insert nodes at the end
-								childs.extend(nodes);
-							}
-						}
-					}
-					if !nexts.is_empty() {
-						// reset nexts index
-						reset_next_siblings_index(index + insert_len, &nexts);
-						// for borrrow check
-						if let Some(parent) = &self.borrow_mut().parent {
-							if let Some(parent) = parent.upgrade() {
-								if let Some(childs) = &mut parent.borrow_mut().childs {
-									//  insert nodes
-									childs.extend(nexts);
-								}
-							}
-						}
-					}
-				}
-				AfterBegin | BeforeEnd => {
-					// set nodes parent
-					for node in &nodes {
-						node.borrow_mut().parent = Some(Rc::downgrade(self));
-					}
-					// prepend, append
-					if let Some(childs) = &mut self.borrow_mut().childs {
-						if *position == BeforeEnd {
-							// reset nodes index
-							reset_next_siblings_index(childs.len(), &nodes);
-							// append nodes
-							childs.extend(nodes);
+			let node_type = node.node_type();
+			let specified: Box<dyn Any> = node.cloned().to_node();
+			if let Ok(dom) = specified.downcast::<RefNode>() {
+				// get the nodes
+				let mut nodes = match node_type {
+					INodeType::DocumentFragement => {
+						if let Some(childs) = &dom.borrow().childs {
+							childs.iter().map(Rc::clone).collect::<Vec<RefNode>>()
 						} else {
-							// always reset nodes index
-							reset_next_siblings_index(0, &nodes);
-							// reset childs index
-							reset_next_siblings_index(nodes.len(), childs);
-							// append childs to nodes
-							nodes.append(childs);
-							// set childs to nodes
-							*childs = nodes;
+							vec![]
 						}
-						return;
 					}
-					// reset nodes index
-					reset_next_siblings_index(0, &nodes);
-					// set nodes as childs
-					self.borrow_mut().childs = Some(nodes);
+					_ => {
+						// remove current node from parent's childs
+						if let Some(parent) = &mut node.parent() {
+							parent.remove_child(node.cloned());
+						}
+						vec![*dom]
+					}
+				};
+				// filter the node allowed
+				let tag_name = self.tag_names();
+				// remove not allowed nodes
+				remove_not_allowed_nodes(&tag_name, &mut nodes);
+				// check if is empty
+				if nodes.is_empty() {
+					return;
 				}
+				// insert
+				use InsertPosition::*;
+				match position {
+					BeforeBegin | AfterEnd => {
+						// get index first, for borrow check
+						let mut index = self.index();
+						let mut nexts: Vec<RefNode> = vec![];
+						let insert_len = nodes.len();
+						// it's insertAfter, increase the insertion index
+						if *position == AfterEnd {
+							index += 1;
+						}
+						// always reset node indexs
+						reset_next_siblings_index(index, &nodes);
+						// split to prev and next
+						if let Some(parent) = &self.borrow_mut().parent {
+							if let Some(parent) = &parent.upgrade() {
+								if let Some(childs) = &mut parent.borrow_mut().childs {
+									// split the nexts for reset index.
+									if index < childs.len() {
+										nexts = childs.split_off(index);
+									}
+									// set node parent
+									for node in &nodes {
+										node.borrow_mut().parent = Some(Rc::downgrade(parent));
+									}
+									// insert nodes at the end
+									childs.extend(nodes);
+								}
+							}
+						}
+						if !nexts.is_empty() {
+							// reset nexts index
+							reset_next_siblings_index(index + insert_len, &nexts);
+							// for borrrow check
+							if let Some(parent) = &self.borrow_mut().parent {
+								if let Some(parent) = parent.upgrade() {
+									if let Some(childs) = &mut parent.borrow_mut().childs {
+										//  insert nodes
+										childs.extend(nexts);
+									}
+								}
+							}
+						}
+					}
+					AfterBegin | BeforeEnd => {
+						// set nodes parent
+						for node in &nodes {
+							node.borrow_mut().parent = Some(Rc::downgrade(self));
+						}
+						// prepend, append
+						if let Some(childs) = &mut self.borrow_mut().childs {
+							if *position == BeforeEnd {
+								// reset nodes index
+								reset_next_siblings_index(childs.len(), &nodes);
+								// append nodes
+								childs.extend(nodes);
+							} else {
+								// always reset nodes index
+								reset_next_siblings_index(0, &nodes);
+								// reset childs index
+								reset_next_siblings_index(nodes.len(), childs);
+								// append childs to nodes
+								nodes.append(childs);
+								// set childs to nodes
+								*childs = nodes;
+							}
+							return;
+						}
+						// reset nodes index
+						reset_next_siblings_index(0, &nodes);
+						// set nodes as childs
+						self.borrow_mut().childs = Some(nodes);
+					}
+				}
+			} else {
+				// not the Dom
+				Dom::halt(
+					self,
+					action,
+					&format!("Can't {} that not implemented 'Dom'", action),
+				);
 			}
-		} else {
-			// not the Dom
-			Dom::halt(
-				self,
-				action,
-				&format!("Can't {} that not implemented 'Dom'", action),
-			);
 		}
 	}
 
-	/// impl `texts`
-	fn texts_by<'b>(
-		&self,
-		limit_depth: usize,
-		handle: &dyn Fn(usize, &BoxDynText) -> bool,
-	) -> Option<Texts<'b>> {
-		let limit_depth = if limit_depth == 0 {
-			usize::MAX
-		} else {
-			limit_depth
-		};
-		let mut result: Texts = Texts::with_capacity(5);
-		fn loop_handle(
-			ele: BoxDynElement,
-			result: &mut Texts,
-			cur_depth: usize,
+	// when the feature `text` is open
+	cfg_feat_text! {
+		/// impl `texts`
+		fn texts_by<'b>(
+			&self,
 			limit_depth: usize,
 			handle: &dyn Fn(usize, &BoxDynText) -> bool,
-		) {
-			let child_nodes = ele.child_nodes();
-			if !child_nodes.is_empty() {
-				let next_depth = cur_depth + 1;
-				let recursive = next_depth < limit_depth;
-				for node in &child_nodes {
-					match node.node_type() {
-						INodeType::Text => {
-							// append text node to result
-							let node = node.clone_node();
-							let text = node.typed().into_text().expect("TextNode must true");
-							if handle(cur_depth, &text) {
-								result.get_mut_ref().push(text);
-							}
-						}
-						INodeType::Element => {
-							let node = node.clone_node();
-							let cur_ele = node.typed().into_element().expect("ElementNode must true");
-							if check_if_content_tag(&cur_ele.tag_names()) {
-								// check if content tag
-								// content tag, change the element into text type
-								let text = cur_ele
-									.into_text()
-									.expect("Content tag must be able to translate into text node");
+		) -> Option<Texts<'b>> {
+			let limit_depth = if limit_depth == 0 {
+				usize::MAX
+			} else {
+				limit_depth
+			};
+			let mut result: Texts = Texts::with_capacity(5);
+			fn loop_handle(
+				ele: BoxDynElement,
+				result: &mut Texts,
+				cur_depth: usize,
+				limit_depth: usize,
+				handle: &dyn Fn(usize, &BoxDynText) -> bool,
+			) {
+				let child_nodes = ele.child_nodes();
+				if !child_nodes.is_empty() {
+					let next_depth = cur_depth + 1;
+					let recursive = next_depth < limit_depth;
+					for node in &child_nodes {
+						match node.node_type() {
+							INodeType::Text => {
+								// append text node to result
+								let node = node.clone_node();
+								let text = node.typed().into_text().expect("TextNode must true");
 								if handle(cur_depth, &text) {
 									result.get_mut_ref().push(text);
 								}
-							} else if recursive {
-								// not content tags need recursive
-								// if need recursive find the text node
-								loop_handle(cur_ele, result, next_depth, limit_depth, handle);
 							}
+							INodeType::Element => {
+								let node = node.clone_node();
+								let cur_ele = node.typed().into_element().expect("ElementNode must true");
+								if check_if_content_tag(&cur_ele.tag_names()) {
+									// check if content tag
+									// content tag, change the element into text type
+									let text = cur_ele
+										.into_text()
+										.expect("Content tag must be able to translate into text node");
+									if handle(cur_depth, &text) {
+										result.get_mut_ref().push(text);
+									}
+								} else if recursive {
+									// not content tags need recursive
+									// if need recursive find the text node
+									loop_handle(cur_ele, result, next_depth, limit_depth, handle);
+								}
+							}
+							_ => {}
 						}
-						_ => {}
+					}
+				} else if check_if_content_tag(&ele.tag_names()) {
+					// content tag, change the element into text type
+					let text = ele
+						.into_text()
+						.expect("Content tag must be able to translate into text node");
+					if handle(cur_depth, &text) {
+						result.get_mut_ref().push(text);
 					}
 				}
-			} else if check_if_content_tag(&ele.tag_names()) {
-				// content tag, change the element into text type
-				let text = ele
-					.into_text()
-					.expect("Content tag must be able to translate into text node");
-				if handle(cur_depth, &text) {
-					result.get_mut_ref().push(text);
-				}
 			}
+			let ele = Box::new(Rc::clone(self)) as BoxDynElement;
+			loop_handle(ele, &mut result, 0, limit_depth, handle);
+			if !result.is_empty() {
+				return Some(result);
+			}
+			None
 		}
-		let ele = Box::new(Rc::clone(self)) as BoxDynElement;
-		loop_handle(ele, &mut result, 0, limit_depth, handle);
-		if !result.is_empty() {
-			return Some(result);
-		}
-		None
 	}
-
 	/// impl `into_text`
 	fn into_text<'b>(self: Box<Self>) -> Result<BoxDynText<'b>, BoxDynError> {
 		if check_if_content_tag(&self.tag_names()) {
